!---------------------------------------------------------------------- 
!----------------------------------------------------------------------
 INCLUDE "equations.f90"	
!---------------------------------------------------------------------- 
!---------------------------------------------------------------------- 

 SUBROUTINE FUNC(NDIM,U,ICP,PAR,IJAC,F,DFDU,DFDP) 
!---------- ---- 
 
  IMPLICIT NONE
  INTEGER, INTENT(IN) :: NDIM, IJAC
  INTEGER, INTENT(IN) :: ICP(*)
  DOUBLE PRECISION, INTENT(IN) :: U(NDIM), PAR(*)
  DOUBLE PRECISION, INTENT(IN) :: DFDU(NDIM,*), DFDP(NDIM,*)
  DOUBLE PRECISION, INTENT(OUT) :: F(NDIM)

    DOUBLE PRECISION T
    INTEGER i

! Get the RHS of the dynamical systems
    CALL GETRHS(NDIM,U,PAR,F)

! Scale
    T = PAR(14)

    DO i=1,NDIM
      F(i) = T * F(i)
    ENDDO
 
 END SUBROUTINE FUNC

!---------------------------------------------------------------------- 

 SUBROUTINE BCND(NDIM,PAR,ICP,NBC,U0,U1,FB,IJAC,DBC) 
!---------- ---- 
 
  IMPLICIT NONE
  INTEGER, INTENT(IN) :: NDIM, NBC, IJAC
  INTEGER, INTENT(IN) :: ICP(*)
  DOUBLE PRECISION, INTENT(IN) :: U0(NDIM), U1(NDIM), PAR(*)
  DOUBLE PRECISION, INTENT(IN) :: DBC(NBC,*)
  DOUBLE PRECISION, INTENT(OUT) :: FB(NDIM)

  INTEGER ND3, ND6, i
  PARAMETER (ND3=3, ND6=6)
  DOUBLE PRECISION ULB(ND6), VLB(ND6), FLB(ND6), eps

! The coordinates of the libration point are in PAR(21)-PAR(23)
    DO i=1,ND3
      ULB(i) = PAR(20+i)
    ENDDO

! Set the velocity components to zero
    DO i=4,ND6
      ULB(i) = 0.d0
    ENDDO

! The equations for the libration point
    CALL GETRHS(ND6,ULB,PAR,FLB)
  
    DO i=1,ND3
      FB(i) = FLB(ND3+i)
    ENDDO

! The unstable eigenvector of the libration point
    CALL EIGLB(ND6,ULB,VLB,PAR)

! The coordinates of the starting point of the orbit in the unstable manifold
    eps = PAR(13)
    DO i=1,ND6
      FB(ND3+i) = U0(i) - ( ULB(i) + eps*VLB(i) )
    ENDDO

! The coordinates of the endpoint of the orbit in the unstable manifold
    DO i=1,ND6
      FB(ND6+ND3+i) = U1(i) - PAR(30+i)
    ENDDO

 END SUBROUTINE BCND

!---------------------------------------------------------------------- 

 SUBROUTINE ICND(NDIM,PAR,ICP,NINT,U,UOLD,UDOT,UPOLD,FI,IJAC,DINT)
!---------- ----

  IMPLICIT NONE
  INTEGER, INTENT(IN) :: NDIM, NINT, IJAC
  INTEGER, INTENT(IN) :: ICP(*)
  DOUBLE PRECISION, INTENT(IN) :: U(NDIM), UOLD(NDIM), UDOT(NDIM), UPOLD(NDIM)
  DOUBLE PRECISION, INTENT(IN) :: DINT(NINT,*), PAR(*)
  DOUBLE PRECISION, INTENT(OUT) :: FI(NDIM)

  INTEGER ND6
  PARAMETER (ND6=6)
  DOUBLE PRECISION F(ND6), T

! The length of the orbit in the manifold is put in PAR(12):

    FI(1) = PAR(14)*DSQRT( U(4)**2 + U(5)**2 + U(6)**2 ) - PAR(12)
!   FI(1) =         DSQRT( U(4)**2 + U(5)**2 + U(6)**2 ) - PAR(12)

 END SUBROUTINE ICND

!---------------------------------------------------------------------- 

SUBROUTINE STPNT(NDIM,U,PAR,time)
!--------- -----

  IMPLICIT NONE
  INTEGER, INTENT(IN) :: NDIM
  DOUBLE PRECISION, INTENT(INOUT) :: U(NDIM), PAR(*)
  DOUBLE PRECISION, INTENT(IN) :: time

  INTEGER ND3, ND6, i
  PARAMETER (ND3=3, ND6=6)
  DOUBLE PRECISION mu, alpha, beta, delta, T, L, eps
  DOUBLE PRECISION ULB(ND6), VLB(ND6)

    mu = 0.5d0
    alpha = 0.d0
    beta  = 0.d0
    delta = 0.d9
    T  = 1d-3
    L  = 1d-3
    eps=-1d-3

    PAR(2) = mu
    PAR(12) = L
    PAR(13) = eps
    PAR(14) = T
    PAR(15) = LOG(mu)
    PAR(16) = alpha
    PAR(17) = beta
    PAR(18) = delta

! Initial libration point
    DO i=1,ND3
      ULB(i) = 0.d0
      ULB(ND3+i) = 0.d0
      PAR(20+i) = ULB(i)
    ENDDO

! The unstable eigenvector of the libration point
    CALL EIGLB(NDIM,ULB,VLB,PAR)

! Initial orbit (taken constant, i.e. time-independent
!                since T is very small)
    DO i=1,NDIM
      U(i) = ULB(i) + eps*VLB(i)
    ENDDO

! Initial end point of orbit (taken equal to the start point)
    DO i=1,NDIM
      PAR(30+i) = U(i)
    ENDDO

 END SUBROUTINE STPNT
!---------------------------------------------------------------------- 

 SUBROUTINE PVLS(NDIM,U,PAR)
!---------- ----

  IMPLICIT NONE
  INTEGER, INTENT(IN) :: NDIM
  DOUBLE PRECISION, INTENT(IN) :: U(NDIM)
  DOUBLE PRECISION, INTENT(INOUT) :: PAR(*)

  DOUBLE PRECISION GETP, mu, r6, i6

! Keep mu updated when log(mu) is used as the continuation parameter
    mu = DEXP(PAR(15))
    PAR(2) = mu

! Put the largest Floquet exponent in PAR(10):

    r6 = GETP('EIG',11,U)
    i6 = GETP('EIG',12,U)
    IF(r6**2 + i6**2 > 0)THEN
       PAR(10) = DLOG10(DSQRT(r6**2 + i6**2))
    ENDIF

!   IF(PAR(10).GT.5.0)WRITE(9,101)PAR(10)
!101 FORMAT("NOTE: log_10(|fm|)=",1PE9.3)

 END SUBROUTINE PVLS
!---------------------------------------------------------------------- 

 SUBROUTINE EIGLB(NDIM,ULB,VLB,PAR)
!---------- -----
 
  IMPLICIT NONE
  INTEGER, INTENT(IN) :: NDIM
  DOUBLE PRECISION, INTENT(IN) :: ULB(NDIM), PAR(*)
  DOUBLE PRECISION, INTENT(OUT) :: VLB(NDIM)

  INTEGER ND6, first, index, i, IER
  DOUBLE PRECISION tolerance
  PARAMETER (ND6=6, tolerance=1.d-6)
  INTEGER IV1(ND6)
  COMMON /FIRST1/ first
     
  DOUBLE PRECISION JAC(ND6,ND6), WR(ND6), WI(ND6), Z(ND6,ND6), FV1(ND6)
  DOUBLE PRECISION rev, iev, ss

! Get the Jacobian at the libration point
    CALL GETJAC(ND6,ULB,PAR,JAC)

! Use the AUTO library routine RG to compute the eigenvalues
    CALL RG(ND6,ND6,JAC,WR,WI,1,Z,IV1,FV1,IER)

    index=0
    DO i=1,ND6
      rev = WR(i)
      iev = WI(i)
      IF(rev.GT.tolerance .AND. DABS(iev).LT.tolerance)THEN
        index=i
        GOTO 1
      ENDIF
    ENDDO

 1  IF(index.NE.0)THEN
      DO i=1,ND6
        VLB(i) = Z(i,index)
      ENDDO
    ELSE
       WRITE(6,100)
       WRITE(9,100)
 100   FORMAT(" ERROR: No real, positive eigenvalue")
       STOP
    ENDIF

! Normalize:
    ss=0.d0
    DO i=1,ND6
      ss = ss + VLB(i)**2
    ENDDO
    ss = DSQRT(ss)
    DO i=1,ND6
      VLB(i) = VLB(i)/ss
    ENDDO

! Set the orientation so the x-component is positive
    IF(VLB(1).LT.0.)THEN
      DO i=1,ND6
        VLB(i) = -VLB(i)
      ENDDO
    ENDIF

! Write the eigenvalues/vectors upon first call:
    IF(first.NE.1234)THEN
      WRITE(9,101)
 101  FORMAT(/," Eigenvalues at the libration point:")
      DO i=1,ND6
        WRITE(9,102)i,WR(i),WI(i)
 102    FORMAT(4X,"Eigenvalue",I2,1P2E19.10)
      ENDDO
      WRITE(9,104)
 104  FORMAT(/," Eigenvector of the real positive eigenvalue:")
      WRITE(9,105)(VLB(i),i=1,ND6)
 105  FORMAT(4X,1P3E19.10)
      first=1234
    ENDIF

 END SUBROUTINE EIGLB
!---------------------------------------------------------------------- 
!---------------------------------------------------------------------- 
 SUBROUTINE FOPT 
 END SUBROUTINE FOPT
!---------------------------------------------------------------------- 
!---------------------------------------------------------------------- 

